#!/usr/bin/perl

%map = (
'Lu' => 'Letter_uppercase',
'Ll' => 'Letter_lowercase',
'Lt' => 'Letter_titlecase',
'Lm' => 'Letter_modifier',
'Lo' => 'Letter_other',
'Mn' => 'Mark_non_spacing',
'Mc' => 'Mark_space_combining',
'Me' => 'Mark_enclosing',
'Nd' => 'Number_decimal_digit',
'Nl' => 'Number_letter',
'No' => 'Number_other',
'Pc' => 'Punctation_connector',
'Pd' => 'Punctation_dash',
'Ps' => 'Punctation_open',
'Pe' => 'Punctation_close',
'Pi' => 'Punctation_initial',
'Pf' => 'Punctation_final',
'Po' => 'Punctation_other',
'Sm' => 'Symbol_math',
'Sc' => 'Symbol_currency',
'Sk' => 'Symbol_modifier',
'So' => 'Symbol_other',
'Zs' => 'Separator_space',
'Zl' => 'Separator_line',
'Zp' => 'Separator_paragraph',
'Cc' => 'Other_control',
'Cf' => 'Other_format',
'Cs' => 'Other_surrogate',
'Co' => 'Other_private_use',
'Cn' => 'Other_not_assigned'
);

%assigned = ();
%page = ();
$last_page = -1;

sub dump_page()
{
  print "static int page$last_page\[256\] = {\n";
  for (my $i = 0; $i < 256; $i++) {
    if (defined $page{$i}) {
      print "\t$page{$i}\n";
    } else {
      print "\t0,\n";
    }
  }
  print "}; /* Page $last_page. */\n\n";

  $assigned{$last_page} = "page$last_page";

  %page = ();
}

while (<>) {
  chomp;
  my ($no, $name, $gencat, $combcat, $bidicat, $decomp, $dec_digit_val,
      $digit_val, $numeric_val, $mirrored, $uc10name, $comment,
      $upper, $lower, $title) = split /;/, $_;

  next if ($name =~ /^\s*<.*(First|Last)\s*>/);
  
  $no =~ /^(.*)(..)$/;
  $page = "0x$1" + 0;
  $char = "0x$2" + 0;

  next if ($page == 0xe00);
  
  if ($page != $last_page) {
    if ($last_page != -1) {
      dump_page();
    }
    $last_page = $page;
  }

  my ($upper_off, $lower_off) = (0,0);

  eval "\$upper_off = - 0x$no + 0x$upper" if ($upper ne "");
  eval "\$lower_off = - 0x$no + 0x$lower" if ($lower ne "");

  $off = 0;
  
  if ($gencat eq "Lu") {
    $off = $lower_off;
  } elsif ($gencat eq "Ll") {
    $off = $upper_off;
  } elsif ($gencat eq "Nd") {
    $off = $dec_digit_val;
  }

  $page{$char} = "U($map{$gencat}, $off),";
 # "\t/* $no: $name */";
}

dump_page();

print "static int *pages[] = {\n";

for (my $i = 0; $i <= $last_page; $i++) {
  if (defined $assigned{$i}) {
    print "\t$assigned{$i},\n";
  } else {
    print "\tNULL,\n";
  }
}

print "};\n\n";

