/* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 944 $ $LastChangedDate: 2002-09-12 22:29:54 +0200 (Thu, 12 Sep 2002) $
 *
 * C implementation for Core module.
 */

#include <gont/runtime.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>

extern gont_int Core$$Match_failure;
extern gont_int Core$$Null_access;
extern gont_int Core$$End_of_file;

void gont_fputs(FILE *f, struct gont_string *s, struct gont_charset *ch)
{
	GONT_STRING_TO_CSTRING(s, ch);

	if (cstring_len && fwrite(cstring, cstring_len, 1, f) != 1)
		gont_sys_error(errno);
}

void Core__puts(struct gont_string *s)
{
	gont_fputs(stdout, s, gont_default_charset_with_approx);
}

void Core__puts_err(struct gont_string *s)
{
	gont_fputs(stderr, s, gont_default_charset_with_approx);
}

/* TODO: fix this to get longer lines. */
struct gont_string *Core__read_line_f(FILE *f)
{
	char buf[4096];
	char *r;

	r = fgets(buf, sizeof(buf), f);

	if (r == NULL)
		gont_raise(gont_make_union(Core$$End_of_file, NULL));
	
	return gont_string_locale(buf);
}

/* TODO: fix this to get longer words. */
struct gont_string *Core__read_word_f(FILE *f)
{
	char buf[4096];

	if (fscanf(f, "%4000s", buf) != 1)
		gont_raise(gont_make_union(Core$$End_of_file, NULL));
	
	return gont_string_locale(buf);
}

struct gont_string *Core__read_line(void)
{
	return Core__read_line_f(stdin);
}

struct gont_string *Core__read_string(void)
{
	return Core__read_word_f(stdin);
}

struct gont_string *Core__itoa(int i)
{
	char buf[100];

	sprintf(buf, "%d", i);
	
	return gont_string_locale(buf);
}

struct gont_string *Core__ftoa(double *f)
{
	char buf[100];

	sprintf(buf, "%g", *f);
	
	return gont_string_locale(buf);
}

void gont_match_failure(void)
{
	gont_raise(gont_make_union(Core$$Match_failure, NULL));
}

void gont_null_access(void)
{
	gont_raise(gont_make_union(Core$$Null_access, NULL));
}

gont_int Core__atoi(struct gont_string *s)
{
	char *p;
	gont_int r;
	
	GONT_STRING_TO_CSTRING(s, gont_default_charset);

	if (cstring_len == 0)
		goto fail;
		
	r = strtol(cstring, &p, 0);

	while (isspace(*p))
		p++;
		
	if (*p != '\0')
		goto fail;
	
	return r;

fail:
	gont_invalid_argument("Core::atoi");
}

double *Core__atof(struct gont_string *s)
{
	char *p;
	double r;

	GONT_STRING_TO_CSTRING(s, gont_default_charset);
	
	if (cstring_len == 0)
		goto fail;
		
	r = strtod(cstring, &p);

	while (isspace(*p))
		p++;
		
	if (*p != '\0')
		goto fail;
	
	return gont_make_float(r);

fail:
	gont_invalid_argument("Core::atof");
	return NULL;	/* never reached */
}

gont_int gont_id(gont_int x)
{
	return x;
}
