/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 1048 $ $LastChangedDate: 2002-10-26 23:19:49 +0200 (Sat, 26 Oct 2002) $
 *
 * Standard library for Gont, Sys module C implementation.
 */

#include <gont/runtime.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/times.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>

int Sys___get_argc(void)
{
	return gont_argc;
}

struct gont_string *Sys___get_argv(gont_int idx)
{
	return gont_string_locale(gont_argv[idx]);
}

int Sys___get_environc(void)
{
	int i;

	if (gont_environ == NULL)
		return 0;
	
	for (i = 0; gont_environ[i]; i++)
		/* nothing */ ;
	
	return i;
}

struct gont_string *Sys___get_environ(gont_int idx)
{
	return gont_string_locale(gont_environ[idx]);
}

void **Sys__times(void)
{
	struct tms t;
	if (times(&t) == (clock_t) -1)
		gont_sys_error(errno);
	return gont_make_tuple(2, (void*) t.tms_utime, (void*)t.tms_stime);
}

gont_int Sys__clk_tck(void)
{
	return sysconf(_SC_CLK_TCK);
}

/* Moved from stdc.c */
void Sys__exit(gont_int x)
{
	gont_run_exit_routines();
	exit(x);
}

void Sys__at_exit(struct gont_lambda_expr *fnc)
{
	gont_register_exit_routine((void (*)(void*))fnc->fnc, fnc->closure);
}

gont_int Sys__shell(struct gont_string *s)
{
	int r;
	GONT_STRING_TO_CSTRING(s, gont_default_charset);

	r = system(cstring);

	if (r == -1)
		gont_sys_error(errno);
	
	return WEXITSTATUS(r);
}

gont_int Sys__isatty(gont_int fd)
{
	return isatty(fd);
}
