/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 1049 $ $LastChangedDate: 2002-10-27 15:32:45 +0100 (Sun, 27 Oct 2002) $
 */

open Ipc;
open Iconv;

extern int _execve(cstring, <cstring>Array::t, <cstring>Array::t) = "Ipc___execve";

int execve(string filename, <string>list argv, <string>list env)
{
    def s2cs(s) (fst(string_to_locale(s)));
    
    _ argv_a = Array::of_list(List::map(s2cs, argv));
    _ env_a = Array::of_list(List::map(s2cs, env));
    Array::append(argv_a, Hacks::cast(0));
    Array::append(env_a, Hacks::cast(0));
    return _execve(s2cs(filename), argv_a, env_a);
}

int execv(string filename, <string>list argv)
{
    return execve(filename, argv, Array::to_list(Sys::environ));
}

extern *(int, int, int) _waitpid(pid_t pid, int options) = "Ipc___waitpid";

wait_status waitpid(pid_t pid, <wait_flag>list flags)
{
    int opts = 0;
    List::iter(fun (s) { switch s {
			 case Wf_no_hang: opts |= 1;
			 case Wf_untraced: opts |= 2; } }, flags);
			 
    let (ret_pid, sel, arg) = _waitpid(pid, opts);
    _ r;
    
    switch sel {
    case 0: r = W_no_children;
    case 1: r = W_exited(arg, ret_pid);
    case 2: r = W_signaled(arg, ret_pid);
    case 3: r = W_signaled_core_dumped(arg, ret_pid);
    case 4: r = W_stopped(arg, ret_pid);
    case _:
	raise Sys_error("Ipc::waitpid(): strange waitpid() result");
    }

    return r;
}

// vim:sts=4:sw=4
