/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 964 $ $LastChangedDate: 2002-09-14 10:19:57 +0200 (Sat, 14 Sep 2002) $
 *
 * Standard library for Gont, Ptrhash module implementation.
 *
 * Mutable hash table.
 */

struct <'a, 'b>t {
	int size;
	int nelem;
	<<*('a, 'b)>list>Array::t tab;
};

// hmm.. we don't really want be dependant on Hashtbl
extern int prime_around(int) = "Hashtbl__prime_around";

<'a, 'b>t create(int sz)
{
	sz = prime_around(sz);

	let ini = fun <*('a, 'b)>list (int) (Nil) in
	return ({ nelem = 0,
		  size = sz,
		  tab = Array::init(sz, ini) });
}

<'a, 'b>t copy(<'a, 'b>t src)
{
	return ({ nelem = src.nelem,
		  size = src.size,
		  tab = Array::copy(src.tab) });
}

int length(<'a, 'b>t h)
{
	return h.nelem;
}

void iter(<'a, 'b>t h, *(void ('a, 'b)) f)
{
	void wrapper(*('a, 'b) p)
	{
		let (a, b) = p in
		f(a, b);
	}
	
	for (int i = 0; i < h.size; i++)
		List::iter(wrapper, Array::get(h.tab, i));
}

void check_rehash(<'a, 'b>t h)
{
	if (h.nelem < h.size)
		return;
		
	<'a, 'b>t n = create(prime_around(h.nelem * 4));

	let f = (fun void ('a k, 'b v) (add(n, k, v))) in
	iter(h, f);

	h.tab = n.tab;
	h.size = n.size;
}

'b find_lst(<*('a, 'b)>list lst, 'a key)
{
	switch lst {
	case Nil:
		raise Not_found;
	case Cons((k, v), tl):
		if (Hacks::ptr_of(k) == Hacks::ptr_of(key))
			return v;
		else
			return find_lst(tl, key);
	}
}

'b find(<'a, 'b>t h, 'a k)
{
	let e = Array::get(h.tab, Hacks::ptr_of(k) % h.size) in
	return find_lst(e, k);
}

<'b>list find_all_lst(<*('a, 'b)>list lst, 'a key)
{
	switch lst {
	case Nil:
		return Nil;
	case Cons((k, v), tl):
		if (Hacks::ptr_of(k) == Hacks::ptr_of(key))
			return Cons(v, find_all_lst(tl, key));
		else
			return find_all_lst(tl, key);
	}
}

<'b>list find_all(<'a, 'b>t h, 'a k)
{
	let e = Array::get(h.tab, Hacks::ptr_of(k) % h.size) in
	return find_all_lst(e, k);
}

bool mem(<'a, 'b>t h, 'a k)
{
	bool xfind(<*('a, 'b)>list lst)
	{
		switch lst {
		case Nil:
			return false;
		case Cons((key, _), tl):
			if (Hacks::ptr_of(k) == Hacks::ptr_of(key))
				return true;
			else
				return xfind(tl);
		}
	}
	
	let e = Array::get(h.tab, Hacks::ptr_of(k) % h.size) in
	return xfind(e);
}

void add(<'a, 'b>t h, 'a k, 'b v)
{
	h.nelem++;
	check_rehash(h);
	let pos = Hacks::ptr_of(k) % h.size in
	let e = Array::get(h.tab, pos) in
	Array::set(h.tab, pos, Cons((k, v), e));
}

void remove(<'a, 'b>t h, 'a k)
{
	<*('a, 'b)>list filter_out(<*('a, 'b)>list lst)
	{
		switch lst {
		case Nil:
			raise Not_found;
		case Cons(hd, tl):
			let (key, _) = hd in
			if (Hacks::ptr_of(k) == Hacks::ptr_of(key)) {
				h.nelem--;
				return tl;
			} else
				return Cons(hd, filter_out(tl));
		}
	}
	
	let idx = Hacks::ptr_of(k) % h.size in
	Array::set(h.tab, idx, filter_out(Array::get(h.tab, idx)));
}

void remove_all(<'a, 'b>t h, 'a k)
{
	bool aint_our_key(*('a, 'b) p)
	{
		let (key, _) = p in 
		if (Hacks::ptr_of(k) != Hacks::ptr_of(key)) {
			h.nelem--;
			return true;
		} else {
			return false;
		}
	}
	
	let idx = Hacks::ptr_of(k) % h.size in
	let l = Array::get(h.tab, idx) in
	Array::set(h.tab, idx, List::filter(aint_our_key, l));
}

// TODO: can be faster
void replace(<'a, 'b>t h, 'a k, 'b v)
{
	remove_all(h, k);
	add(h, k, v);
}

def keys(h)
{
	_ ret = [];
	
	def add(k, _)
	{
		ret = Cons(k, ret);
	}

	iter(h, add);

	return ret;
}

def values(h)
{
	_ ret = [];
	
	def add(_, v)
	{
		ret = Cons(v, ret);
	}

	iter(h, add);

	return ret;
}

def of_assoc_list(lst)
{
	_ ret = create(List::length(lst));
	List::iter(fun (p) {let (k, v) = p in add(ret, k, v);}, lst);
	return ret;
}
