/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 666 $ $LastChangedDate: 2002-08-29 10:59:13 +0200 (Thu, 29 Aug 2002) $
 *
 * Standard library for Gont, Unionfind module implementation.
 */

struct <'b>top {
    'b set;
    int count;
}

union <'b>link {
    <'b>set Uplink;
    <'b>top Top;
}

struct <'a>set {
    <'b>link link;
}

struct <'a,'b>t {
    <'a, <'b>set>Ptrhash::t elts;
    <'b, <'b>set>Ptrhash::t sets;
}

<'a,'b>t create()
{
    return create_ex(13, 13);
}

<'a,'b>t create_ex(int elts_size_hint, int sets_size_hint)
{
    return {elts = Ptrhash::create(elts_size_hint),
	    sets = Ptrhash::create(sets_size_hint)};
}

void make_set(<'a,'b>t h, 'b s)
{
    if (Ptrhash::mem(h.sets, s))
	raise Invalid_argument("Unionfind::make_set");
    Ptrhash::add(h.sets, s, {link = Top({set = s, count = 0})});
}

*(<'b>top, <'b>set) find_set(<'b>set s)
{
    switch s.link {
    case Top(t):
	return (t, s);
    case Uplink(u):
	// path compression
	let (t, st) = find_set(u) in {
	    s.link = Uplink(st);
	    return (t, st);
	}
    }
}

void insert(<'a,'b>t h, 'a e, 'b s)
{
    if (Ptrhash::mem(h.elts, e))
	raise Invalid_argument("Unionfind::insert");
    let (t, st) = find_set(Ptrhash::find(h.sets, s)) in {
	t.count++;
	Ptrhash::add(h.elts, e, st);
    }
}

void union_sets(<'a,'b>t h, 'b dst, 'b src)
{
    let (td, d) = find_set(Ptrhash::find(h.sets, dst)) in
    let (ts, s) = find_set(Ptrhash::find(h.sets, src)) in {
	if (Hacks::same_ptrs(td, ts))
	    return;
	if (td.count > ts.count) {
	    s.link = Uplink(d);
	    td.count = td.count + ts.count;
	} else {
	    d.link = Uplink(s);
	    ts.count = td.count + ts.count;
	    ts.set = td.set;
	}
    }
}

'b find(<'a,'b>t h, 'a e)
{
    let (t, _) = find_set(Ptrhash::find(h.elts, e)) in
    return t.set;
}

// vim:sts=4:sw=4
